@@ -5,7 +5,6 @@ Takes ESA zipped datasets or Sinergise dataset directories
 """
 import fnmatch
 import json
-import logging
 import os.path
 import re
 import sys
@@ -18,16 +17,17 @@ from pathlib import Path
 from typing import Dict, Iterable, List, Mapping, Optional, Tuple, Union
 
 import click
+import structlog
 from attr import define
-from defusedxml import minidom
-
 from datacube.config import LocalConfig
 from datacube.index import index_connect
 from datacube.model import Dataset
 from datacube.ui.click import config_option, environment_option
 from datacube.utils.uris import normalise_path
+from defusedxml import minidom
+
 from eodatasets3 import DatasetDoc, DatasetPrepare, names, serialise
-from eodatasets3.prepare.s2_common import RegionLookup, FolderInfo
+from eodatasets3.prepare.s2_common import FolderInfo, RegionLookup
 from eodatasets3.properties import Eo3Interface
 from eodatasets3.ui import PathPath
 from eodatasets3.utils import pass_config
@@ -35,7 +35,7 @@ from eodatasets3.utils import pass_config
 # Static namespace to generate uuids for datacube indexing
 S2_UUID_NAMESPACE = uuid.UUID("9df23adf-fc90-4ec7-9299-57bd536c7590")
 
-_LOG = logging.getLogger("sentinel-l1")
+_LOG = structlog.get_logger()
 
 SENTINEL_MSI_BAND_ALIASES = {
     "01": "coastal_aerosol",
@@ -127,6 +127,15 @@ def process_tile_metadata(contents: str) -> Dict:
     resolution = min(
         int(i.attributes["resolution"].value) for i in root.getElementsByTagName("Size")
     )
+    tile_id = _value(root, "TILE_ID")
+
+    region_code = tile_id.split("_")[-2]
+    if not region_code.startswith("T"):
+        raise RuntimeError(
+            f"Tile id is not recognised -- not a region code? Please report this. {tile_id!r}"
+        )
+    region_code = region_code[1:]
+
     return {
         "datetime": _value(root, "SENSING_TIME"),
         "eo:cloud_cover": _value(root, "CLOUDY_PIXEL_PERCENTAGE", type_=float),
@@ -135,7 +144,8 @@ def process_tile_metadata(contents: str) -> Dict:
         "eo:sun_elevation": _value(root, "Mean_Sun_Angle", "ZENITH_ANGLE", type_=float),
         "odc:processing_datetime": _value(root, "ARCHIVING_TIME"),
         "sentinel:datastrip_id": _value(root, "DATASTRIP_ID"),
-        "sentinel:sentinel_tile_id": _value(root, "TILE_ID"),
+        "sentinel:sentinel_tile_id": tile_id,
+        "odc:region_code": region_code,
     }
 
 
@@ -150,7 +160,9 @@ def process_datastrip_metadata(contents: str) -> Dict:
     resolution_tags = root.getElementsByTagName("RESOLUTION")
     if resolution_tags:
         resolution = min(int(i.firstChild.data) for i in resolution_tags)
-    elif root.getElementsByTagName("SPACECRAFT_NAME")[0].firstChild.data.startswith("Sentinel-2"):
+    elif root.getElementsByTagName("SPACECRAFT_NAME")[0].firstChild.data.startswith(
+        "Sentinel-2"
+    ):
         resolution = 10.0
     else:
         raise ValueError("No resolution in datastrip metadata and unknown craft")
@@ -175,7 +187,6 @@ def process_user_product_metadata(contents: str, filename_stem: str = None) -> D
     else:
         product_uri = filename_stem
 
-    region_code = product_uri.split("_")[5][1:]
     return {
         "eo:platform": _value(root, "SPACECRAFT_NAME"),
         "sat:relative_orbit": _value(root, "SENSING_ORBIT_NUMBER", type_=int),
@@ -184,7 +195,6 @@ def process_user_product_metadata(contents: str, filename_stem: str = None) -> D
         "sentinel:processing_baseline": _value(root, "PROCESSING_BASELINE"),
         "sentinel:product_name": product_uri,
         "eo:cloud_cover": _value(root, "Cloud_Coverage_Assessment"),
-        "odc:region_code": region_code,
     }
 
 
@@ -232,11 +242,10 @@ def prepare_and_write(
         # Default to embedding the location if they're not in the same folder.
         embed_location = output_yaml.parent not in dataset_location.parents
         _LOG.debug(
-            "Auto-embed location? %s: %s %s %s",
-            "Yes" if embed_location else "No",
-            dataset_location.parent,
-            "!=" if embed_location else "==",
-            output_yaml.parent,
+            "Auto-embed location?",
+            auto_embed=bool(embed_location),
+            data_location=dataset_location.parent,
+            yaml_location=output_yaml.parent,
         )
 
     with DatasetPrepare(
@@ -346,7 +355,7 @@ def _extract_esa_fields(
             internal_folder_name = os.path.commonprefix(z.namelist())
             for s in z.namelist():
                 if any(
-                    fnmatch.fnmatch(s[len(internal_folder_name) :], pattern)
+                    re.match(pattern, s[len(internal_folder_name) :])
                     for pattern in patterns
                 ):
                     yield s
@@ -362,7 +371,7 @@ def _extract_esa_fields(
                 )
             return matches[0]
 
-        datastrip_md = one("*MTD_DS.xml", "DATASTRIP/S2*.xml")
+        datastrip_md = one(r".*MTD_DS\.xml$", r"DATASTRIP/S2[^/]+/S2[^/]+\.xml$")
         p.properties.update(
             process_datastrip_metadata(z.read(datastrip_md).decode("utf-8"))
         )
@@ -370,8 +379,8 @@ def _extract_esa_fields(
 
         # Get the specific granule metadata
         [*tile_mds] = find(
-            "*MTD_TL.xml",
-            f"GRANULE/{granule_id}/S2*.xml" if granule_id else "GRANULE/S2*.xml",
+            r".*MTD_TL\.xml$",
+            rf"GRANULE/{granule_id}/S2.*\.xml" if granule_id else r"GRANULE/S2.*\.xml",
         )
         if not tile_mds:
             raise ValueError(
@@ -393,7 +402,7 @@ def _extract_esa_fields(
         p.note_accessory_file("metadata:s2_tile", tile_md)
 
         # Wider product metadata.
-        user_product_md = one("*MTD_MSIL1C.xml", "S2*.xml")
+        user_product_md = one(r".*MTD_MSIL1C\.xml", r"S2.*\.xml")
         for prop, value in process_user_product_metadata(
             z.read(user_product_md).decode("utf-8"),
             filename_stem=Path(user_product_md).stem,
@@ -621,9 +630,6 @@ def main(
             "Command name 'sentinel-l1c-prepare' is deprecated: remove the 'c', and use `sentinel-l1-prepare`"
         )
 
-    logging.basicConfig(format="%(asctime)s %(levelname)s %(message)s")
-    _LOG.setLevel(logging.DEBUG if verbose else logging.INFO)
-
     included_regions = None
     if only_regions_in_file:
         included_regions = set(only_regions_in_file.read_text().splitlines())
@@ -637,14 +643,12 @@ def main(
             ),
         ]
 
-    _LOG.info(f"{len(datasets)} paths(s) to process using {workers} worker(s))")
+    _LOG.info("kickoff", path_count=len(datasets), worker_count=workers)
 
     # Are we indexing on success?
     index = None
     if index_to_odc:
-        _LOG.info("Indexing new datasets")
-        if local_config:
-            _LOG.debug("Indexing to %s", local_config)
+        _LOG.info("Indexing new datasets", local_config=local_config)
         index = index_connect(local_config, application_name="s2-prepare")
         products = {}
 
@@ -663,7 +667,9 @@ def main(
             index.datasets.add(
                 Dataset(product, serialise.to_doc(dataset), uris=dataset.locations)
             )
-            _LOG.debug("Indexed dataset %s to %s", dataset.id, dataset_path)
+            _LOG.debug(
+                "Indexed dataset", dataset_id=dataset.id, dataset_path=dataset_path
+            )
 
     else:
 
@@ -712,7 +718,7 @@ def main(
 
             first = True
             for found_dataset in find_inputs_in_path(input_path):
-                _LOG.debug(f"Trying {found_dataset.name}")
+                _LOG.debug("found_dataset", name=found_dataset.name)
                 # Make sure we tick progress on extra datasets that were found.
                 if not first:
                     first = False
@@ -733,16 +739,22 @@ def main(
                         if info.region_code is None:
                             for region in region_lookup.get(info.area):
                                 if region in included_regions:
-                                    _LOG.debug(f'Area {info.area!r} mapped to (included) region {region!r}')
+                                    _LOG.debug(
+                                        "mapped_area_match",
+                                        input_area=info.area,
+                                        region_match=region,
+                                    )
                                     break
                             else:
                                 _LOG.debug(
-                                    f"Skipping because area {info.area!r} doesn't map to an included region"
+                                    "skipping.mapped_area_not_in_regions",
+                                    input_area=info.area,
                                 )
                                 continue
                         elif info.region_code not in included_regions:
                             _LOG.debug(
-                                f"Skipping because region {info.region_code!r} is not in region list"
+                                "skipping.region_not_in_region_list",
+                                region_code=info.region_code,
                             )
                             continue
 
@@ -753,7 +765,9 @@ def main(
                             info.year == year and info.month < month
                         ):
                             _LOG.debug(
-                                f"Skipping because year {info.year}-{info.month} is older than {year}-{month}"
+                                "skipping.too_old",
+                                dataset_year_month=(info.year, info.month),
+                                max_year_month=(year, month),
                             )
                             continue
                     if before_month is not None:
@@ -763,7 +777,9 @@ def main(
                             info.year == year and info.month > month
                         ):
                             _LOG.debug(
-                                f"Skipping because year {info.year}-{info.month} is newer than {year}-{month}"
+                                "skipping.too_young",
+                                dataset_year_month=(info.year, info.month),
+                                min_year_month=(year, month),
                             )
                             continue
 
@@ -801,7 +817,8 @@ def main(
                 ):
                     # Skip it!
                     _LOG.debug(
-                        "At least one output exists: skipping. %s", found_dataset.name
+                        "At least one output exists: skipping.",
+                        dataset_name=found_dataset.name,
                     )
                     continue
 
@@ -812,7 +829,7 @@ def main(
                 if not granule_ids:
                     granule_ids = [None]
                 else:
-                    _LOG.debug(f"Found {len(granule_ids)} granules")
+                    _LOG.debug("found_granules", granule_count=len(granule_ids))
 
                 for granule_id in granule_ids:
                     if always_granule_id or (
@@ -829,13 +846,19 @@ def main(
                     output_yaml = output_folder / yaml_filename
                     if output_yaml.exists():
                         if not overwrite_existing:
-                            _LOG.debug("Output exists: skipping. %s", output_yaml)
+                            _LOG.debug(
+                                "Output exists: skipping.", output_yaml=output_yaml
+                            )
                             continue
 
-                        _LOG.debug("Output exists: overwriting %s", output_yaml)
+                        _LOG.debug(
+                            "Output exists: overwriting.", output_yaml=output_yaml
+                        )
 
                     _LOG.info(
-                        f'Queued {found_dataset.name} (granule: {granule_id or "assuming-single"})'
+                        "queued",
+                        dataset_name=found_dataset.name,
+                        granule=granule_id or "any",
                     )
                     yield Job(
                         dataset_path=found_dataset.path,
@@ -860,9 +883,9 @@ def main(
                 try:
                     if dry_run:
                         _LOG.info(
-                            "Would write dataset %s to %s",
-                            job.dataset_path,
-                            job.output_yaml_path,
+                            "Would write dataset",
+                            dataset_path=job.dataset_path,
+                            output_yaml_path=job.output_yaml_path,
                         )
                     else:
                         dataset, path = prepare_and_write(
@@ -872,11 +895,13 @@ def main(
                             granule_id=job.granule_id,
                             embed_location=job.embed_location,
                         )
-                        _LOG.info("Wrote dataset %s to %s", dataset.id, path)
+                        _LOG.info(
+                            "Wrote dataset", dataset_id=dataset.id, datsaset_path=path
+                        )
                         on_success(dataset, path)
                     successes += 1
                 except Exception:
-                    _LOG.exception("Failed to complete dataset: %s", job)
+                    _LOG.exception("failed_job", job=job)
                     errors += 1
         else:
             with Pool(processes=workers) as pool:
@@ -886,7 +911,9 @@ def main(
                         errors += 1
                     else:
                         dataset, path = res
-                        _LOG.info("Wrote dataset %s to %s", dataset.id, path)
+                        _LOG.info(
+                            "Wrote dataset", dataset_id=dataset.id, dataset_path=path
+                        )
                         on_success(dataset, path)
                         successes += 1
                 pool.close()
@@ -895,9 +922,7 @@ def main(
         if index is not None:
             index.close()
 
-    _LOG.info(
-        f"Completed {successes} dataset(s) successfully, with {errors} failure(s)"
-    )
+    _LOG.info("completed", success_count=successes, failure_count=errors)
     sys.exit(errors)
 
 
@@ -905,7 +930,7 @@ def _get_default_relative_folder_base(path: Path) -> Optional[Path]:
     for parent in path.parents:
         if parent.name.lower() == "l1c":
             input_relative_to = parent
-            _LOG.debug(f"Inputs will be relative to {input_relative_to}")
+            _LOG.debug("found_base_folder", inputs_are_relative_to=input_relative_to)
             break
     else:
         raise ValueError(
